package de.hsh.dbs2.imdb.records;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

public class Movie implements ActiveRecord {
	private Long movieId;
	private String title;
	private int year;
	private char type;
	
	private static Counter counter;
	
	public Movie(Long movieId, String title, int year, char type) throws SQLException {
		this.movieId = movieId;
		this.title = title;
		this.year = year;
		this.type = type;
		
		if (counter == null) {
			counter = Counter.get("Movie");
		}
	}

	public String getTitle() {
		return title;
	}

	public void setTitle(String title) {
		this.title = title;
	}

	public int getYear() {
		return year;
	}

	public void setYear(int year) {
		this.year = year;
	}

	public char getType() {
		return type;
	}

	public void setType(char type) {
		this.type = type;
	}

	public Long getMovieID() {
		return movieId;
	}
	
	@Override
	public String toString() {
		return "Movie: " + this.movieId + " : " + this.title + " : " + this.year + " : " + this.type;
	}

	@Override
	public void insert() throws SQLException {
		if (this.getMovieID() == null) {
			this.movieId = counter.next();
		}
		String sql = "INSERT INTO `Movie` VALUES (?, ?, ?, ?);";
		try (PreparedStatement ps = ConnectionManager.getConnection().prepareStatement(sql)) {
			ps.setLong(1, this.movieId);
			ps.setString(2, this.title);
			ps.setInt(3, this.year);
			ps.setString(4, "" + this.type);
			ps.executeUpdate();
		}
	}

	@Override
	public void update() throws SQLException {
		String sql = "UPDATE `Movie` SET `Title`=?, `Year`=?, `Type`=? WHERE `MovieID`=?";
		try (PreparedStatement ps = ConnectionManager.getConnection().prepareStatement(sql)) {
			ps.setString(1,  this.title);
			ps.setInt(2,  this.year);
			ps.setString(3, "" + this.type);
			ps.setLong(4,  this.movieId);
			ps.executeUpdate();
		}
		
	}

	@Override
	public void delete() throws SQLException {
		String sql = "DELETE FROM `Movie` WHERE `MovieID`=?";
		try (PreparedStatement ps = ConnectionManager.getConnection().prepareStatement(sql)) {
			ps.setLong(1,  this.movieId);
			ps.executeUpdate();
		}
	}
	
	
	public static ArrayList<Movie> findAll() throws SQLException {
		String sql = "SELECT * FROM Movie";
		ArrayList<Movie> movies = new ArrayList<Movie>();
		try (PreparedStatement ps = ConnectionManager.getConnection().prepareStatement(sql)) {
			try (ResultSet rs = ps.executeQuery()) {
				int i = 0;
				while (rs.next()) {
					movies.add(new Movie(rs.getLong("MovieID"), rs.getString("Title"), rs.getInt("Year"), rs.getString("Type").charAt(0)));
				}
			}
		}
		return movies;
	}
	
	public static Movie findById(Long movieId) throws SQLException {
		String sql = "SELECT * FROM `Movie` WHERE `MovieID`=?";
		try (PreparedStatement ps = ConnectionManager.getConnection().prepareStatement(sql)) {
			ps.setLong(1, movieId);
			try (ResultSet rs = ps.executeQuery()) {
				if (!rs.isBeforeFirst()) return null;
				return new Movie(rs.getLong("MovieID"), rs.getString("Title"), rs.getInt("Year"), rs.getString("Type").charAt(0));
			}
		}
	}
	
	
}
