package de.hsh.dbs2.imdb.records;
import java.sql.*;
import java.util.ArrayList;

public class Person implements ActiveRecord {
	private Long personId;
	private String name;
	private char sex;
	
	private static Counter counter;
	
	public Person(Long personId, String name, char sex) throws SQLException {
		this.personId = personId;
		this.name = name;
		this.sex = sex;
		
		if (counter == null) {
			counter = Counter.get("Person");
		}
	}
	
	public Long getPersonID() {
		return personId;
	}
	
	public String getName() {
		return name;
	}
	
	public char getSex() {
		return sex;
	}
	
	public void setName(String name) {
		this.name = name;
	}
	
	public void setSex(char sex) {
		this.sex = sex;
	}

	@Override
	public void insert() throws SQLException {
		if (this.getPersonID() == null) {
			this.personId = counter.next();
		}
		String sql = "INSERT INTO `Person` VALUES (?, ?, ?);";
		try (PreparedStatement ps = ConnectionManager.getConnection().prepareStatement(sql)) {
			ps.setLong(1,  this.personId);
			ps.setString(2,  this.name);
			ps.setString(3, "" + this.sex);
			ps.executeUpdate();
		}
	}

	@Override
	public void update() throws SQLException {
		String sql = "UPDATE `Person` SET `Name`=?, `Sex`=? WHERE `PersonID`=?";
		try (PreparedStatement ps = ConnectionManager.getConnection().prepareStatement(sql)) {
			ps.setString(1,  this.name);
			ps.setString(2, "" + this.sex);
			ps.setLong(3,  this.personId);
			ps.executeUpdate();
		}
	}

	@Override
	public void delete() throws SQLException {
		String sql = "DELETE FROM `Person` WHERE `PersonID`=?";
		try (PreparedStatement ps = ConnectionManager.getConnection().prepareStatement(sql)) {
			ps.setLong(1,  this.personId);
			ps.executeUpdate();
		}
	}
	
	public String toString() {
		return "Person: " + this.personId + " : " + this.name + " : " + this.sex;
	}
	
	public static ArrayList<Person> findAll() throws SQLException {
		String sql = "SELECT * FROM Person";
		ArrayList<Person> personen = new ArrayList<Person>();
		try (PreparedStatement ps = ConnectionManager.getConnection().prepareStatement(sql)) {
			try (ResultSet rs = ps.executeQuery()) {
				int i = 0;
				while (rs.next()) {
					personen.add(new Person(rs.getLong("PersonID"), rs.getString("Name"), rs.getString("Sex").charAt(0)));
				}
			}
		}
		return personen;
	}
	
	public static Person findById(Long personId) throws SQLException {
		String sql = "SELECT * FROM `Person` WHERE `PersonId`=?";
		try (PreparedStatement ps = ConnectionManager.getConnection().prepareStatement(sql)) {
			ps.setLong(1, personId);
			try (ResultSet rs = ps.executeQuery()) {
				if (!rs.isBeforeFirst()) return null;
				return new Person(rs.getLong("PersonID"), rs.getString("Name"), rs.getString("Sex").charAt(0));
			}
		}
	}

	public static Person findByName(String name) throws SQLException {
		String sql = "SELECT * FROM `Person` WHERE `Name`=?";
		try (PreparedStatement ps = ConnectionManager.getConnection().prepareStatement(sql)) {
			ps.setString(1, name);
			try (ResultSet rs = ps.executeQuery()) {
				if (!rs.isBeforeFirst()) return null;
				return new Person(rs.getLong("PersonID"), rs.getString("Name"), rs.getString("Sex").charAt(0));
			}
		}
	}
}
